<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SM\Service\Sm00009000ParamFaturaModeloContratoService;

class Sm00009000ParamFaturaModeloContratoController extends SMCrud {
    
    protected $dataSourceName = 'fct_paramfatmodcont_widget';
    protected $ServiceParamFaturaModeloContrato;
    
    public function __construct ($crudService, $environment, Sm00009000ParamFaturaModeloContratoService $ServiceParamFaturaModeloContrato) {
        parent::__construct($crudService, $environment);
        $this->ServiceParamFaturaModeloContrato = $ServiceParamFaturaModeloContrato;
    }

    public function procuraPerfilParametrizado (Request\Row $request, Response $response) {
        $row = $request->getRow();
        $nrorg   = $row['NRORG'];
        $isproporcont = $row["PROPOUCONT"];
        if($isproporcont == 'C'){
            $nrmodelo  = $row['NRSEQMODCONTRA'];
        }
        if($isproporcont == 'P'){
            $nrmodelo  = $row['NRORCAMENTOMODELO'];
        }

        $result = $this->ServiceParamFaturaModeloContrato->procuraPerfilParametrizado($nrorg, $nrmodelo, $isproporcont);
        $response->setDataSets(array(new DataSet("validacao", $result)));

    }
   
    /**
     * A função "insereDadosModContrato" insere os dados de parametização 
     * de fatiuramento na tabela de Modelo de Contrato (FCT_MODELOCONTRATO).
     */
    
    public function insereDadosModContrato(Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        $dtInicFatContr   = $row['DTINICFATCONTR'];
        $idQtDiasCompCon  = $row['IDQTDIASCOMPCON'];
        $nrQtDiasCompCon  = $row['NRQTDIASCOMPCON'];
        $idCompetCont     = $row['IDTPCOMPETCONT'];
        $nrDiaCompetCont  = $row['NRDIACOMPETCONT'];
        $idTpDtEmissNf    = $row['IDTPDTEMISSNF'];
        $nrDiaDtEmissNf   = $row['NRDIADTEMISSNF'];
        $idTpDtVenc       = $row['IDTPDTVENC'];
        $nrDiaVenParcFat  = $row['NRDIAVENPARCFAT'];
        $idTpFatuIncomp   = $row['IDTPFATUINCOMP'];
        $seqModContrato   = $row['NRSEQMODCONTRA'];
        
        $InsereDadosContr = $this->ServiceParamFaturaModeloContrato->insereDadosModContrato($dtInicFatContr, $idQtDiasCompCon, $nrQtDiasCompCon, $idCompetCont, $nrDiaCompetCont, $idTpDtEmissNf, $nrDiaDtEmissNf, $idTpDtVenc, $nrDiaVenParcFat, $idTpFatuIncomp, $seqModContrato);
    }
    
    /**
     * insere os signatários padrões de modelo de contrato que foram buscados na parametrização SM 
    **/
    public function insereSignatariosPadroesContr(Request\Row $request, Response $response){
        try {
            $row      = $request->getRow();
            $nrorg    = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
            
            $this->ServiceParamFaturaModeloContrato->insereSignatariosPadroesContr($row, $nrorg, $operador);
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * A função "cadAutoSignatPadrCont" realiza o cadastramento automático dos signatários padrões do modelo de contrato
     */
    public function cadAutoSignatPadrCont(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $result = $this->ServiceParamFaturaModeloContrato->cadAutoSignatPadrCont($row);
            $response->addDataSet(new DataSet("cadauto", array("retorno" => $result)));

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * A função "validaSignatPadrCont" realiza a validação dos signatários padrões para evitar a inclusão repetida de um parceiro.
     */
    public function validaSignatPadrCont(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $result = $this->ServiceParamFaturaModeloContrato->validaSignatPadrCont($row);
            $response->addDataSet(new DataSet("validaSignatPadrCont", array("valida" => $result)));

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     /**
     * Valida se o token da certsign está preenchido
     */
    public function validaIdCertsign(Request\Row $request, Response $response){
        try {
            $nrorg = $this->environment->getNrorg();
            
            $result = $this->ServiceParamFaturaModeloContrato->validaIdCertsign($nrorg);
            $response->addDataSet(new DataSet("validaIdCertsign", array("retorno" => $result)));
            
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
        /**
     * A função "insereOperadoresNotifica" realiza a inserção dos operadores do sistema que devem ser notificados.
     */
    public function insereOperadoresNotifica(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $nrsignatpadrcont = $row["NRSIGNATPADRCONT"];
            $operadores = $row["OPERADORES"];
            
            $this->ServiceParamFaturaModeloContrato->insereOperadoresNotifica($nrsignatpadrcont, $operadores);

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}

?>