<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Teknisa\SM\Util\Utilities;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SM\Service\Sm00010000FaturamentoApuracaoService;
use Teknisa\SM\Controller\BirtReport;
use Zeedhi\Framework\DTO\Row;
use Teknisa\SM\Service\Sm00005000ParametrizacoesServicesService;

class Sm00010000FaturamentoApuracaoController extends SMCrud
{
    protected $dataSourceName = "fct_faturacont_completo";
    protected $apuracaoFaturamentoService;
    protected $birtController;
    protected $parametrogeral;
    
    public function __construct($crudService, $environment, Sm00010000FaturamentoApuracaoService $apuracaoFaturamentoService, BirtReport $birtController, Sm00005000ParametrizacoesServicesService $parametrogeral) {
        parent::__construct($crudService, $environment);
        $this->apuracaoFaturamentoService = $apuracaoFaturamentoService;
        $this->birtController             = $birtController;
        $this->parametrogeral             = $parametrogeral;
    }

    public function find(Request\Filter $request, Response $response){
        try {
            $filterCriteria = $request->getFilterCriteria();
            $conditions = $filterCriteria->getConditions();
            
            $newConditions = array();
            
            $contratos = null;
            
            foreach ($conditions as $condition) {
                
                if ($condition["columnName"] == "NRSEQCONTRATO") {
                    
                    $contratos = implode('_', $condition["value"]);
                    $contratos = '_'.$contratos.'_';
                    
                    $newConditions[] =  array(
                        "columnName" => "NRSEQCONTRATO",
                        "value" => $contratos,
                        "operator" => '='
                    );
                }
                else {
                    $newConditions[] =  $condition;
                }
            }
            
            if ($contratos != null) {
                $newFilterCriteria = new FilterCriteria($filterCriteria->getDataSourceName(), $newConditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());
                $this->crudService->keepOrderBy($filterCriteria, $newFilterCriteria);
                $newrequest = new Request\Filter($newFilterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                parent::find($newrequest, $response);
            }
            else {
                parent::find($request, $response);  
            }
        }
        catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function invoiceValidateHasAlreadyBeenUsed (Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrLanctoNF    = $row["NRLANCTONF"];
            $cdFilial      = $row["CDFILIAL"];
            $dataSet  = $this->apuracaoFaturamentoService->invoiceValidateHasAlreadyBeenUsed($nrLanctoNF,$cdFilial);
            Utilities::setDataSetDefault($request,$response,$dataSet) ;
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * A função "faturarParcelas" Realiza o Faturamento das parcelas marcadas.
     */
    public function faturarParcelas(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrSeqContrato    = $row["P_NRSEQCONTRATO"];
            $nrParcNegocio    = $row["P_NRPARCNEGOCIO"];
            $dtInicial        = $row["P_DATAINI"];
            $dtFinal          = $row["P_DATAFIN"];
            $nrVencPrevCont   = $row["P_NRVENCPREVCONT"];
            $nrGrupoFatura    = $row["P_NRGRUPOFATURA"];
            $nrrateioitemvenc = $row["P_NRRATEIOITEMFAT"];
            
            $nrorg    = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();
            // -- Gerar Parcelas de Faturamento -- 
            $this->apuracaoFaturamentoService->executarFctFaturaContrato($nrSeqContrato, $nrParcNegocio, $dtInicial, $dtFinal, $nrVencPrevCont, $nrGrupoFatura, $nrrateioitemvenc, $nrorg, $operador);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Atualiza o valor do vencimento de acordo com a composição de serviço da proposta e os tipos de medida variáveis.
     */
    public function atualizarValoresVencimento(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrvencprevcont = $row["NRVENCPREVCONT"];
            $nrseqcontrato  = $row["NRSEQCONTRATO"];
            $medidas        = $row["TIPOS_MEDIDA_VARIAVEIS"];
            $nrorg          = $this->environment->getNrOrg();
            
            $this->apuracaoFaturamentoService->atualizarValoresVencimento($nrvencprevcont, $nrseqcontrato, $nrorg, $medidas);
            
            if (isset($row["HORAS"]) && is_array($row["HORAS"])) {
                $horasAgendadas = $row["HORAS"]["agendadas"];
                $horasApontadas = $row["HORAS"]["apontadas"];
                
                $valorVencimento = $this->apuracaoFaturamentoService->getValorVencimento($nrvencprevcont, $nrseqcontrato, $nrorg);
                $valorNovo = $valorVencimento * $horasApontadas / $horasAgendadas;
                $this->apuracaoFaturamentoService->atualizarValorTotalVencimento($valorNovo, $nrvencprevcont, $nrseqcontrato, $nrorg);
            }
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Retorna os tipos de medida variáveis da proposta que gerou o contrato.
     */
    public function getMedidasVariaveisDaPropostaDoContrato(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $dtinicompet   = $row["DTINICOMPET"];
            $dtfincompet   = $row["DTFINCOMPET"];
            
            $nrorg = $this->environment->getNrOrg();
            
            $medidas = $this->apuracaoFaturamentoService->getMedidasVariaveisDaPropostaDoContrato($nrseqcontrato, $nrorg);
            $response->addDataSet(new DataSet("medidas", $medidas));
            
            /*$apuraApontamentoHoras = $this->apuracaoFaturamentoService->verificarApuraApontamentoHoras($nrseqcontrato, $nrorg);
            if ($apuraApontamentoHoras) {
                $horasAgendadas = $this->apuracaoFaturamentoService->getTotalHorasAgendadasContrato($nrseqcontrato, $dtinicompet, $dtfincompet, $nrorg);
                $horasApontadas = $this->apuracaoFaturamentoService->getTotalHorasApontadasContrato($nrseqcontrato, $dtinicompet, $dtfincompet, $nrorg);
                
                $horas = array(
                    "agendadas" => $horasAgendadas,
                    "apontadas" => $horasApontadas
                );
                
                $response->addDataSet(new DataSet("horas", $horas));
            }*/
            
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Cancela a apuração do faturamento.
     */
    public function cancelarFaturamentoVencimentos(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrseqcontrato    = $row["P_NRSEQCONTRATO"];
            $dataini          = $row["P_DATAINI"];
            $datafin          = $row["P_DATAFIN"];
            $nrvencprevcont   = $row["P_NRVENCPREVCONT"];
            $nrrateioitemvenc = $row["P_NRRATEIOITEMFAT"];
            
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $this->apuracaoFaturamentoService->executarProcedureFctCancFaturaContrato($nrseqcontrato, $dataini, $datafin, $nrvencprevcont, $nrrateioitemvenc, $nrorg, $cdoperador);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Cancela a apuração do faturamento para o tipo "Apuração".
     */
    public function cancelarFaturamentoVencimentoTipoApuracao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrseqcontrato  = $row["NRSEQCONTRATO"];
            $nrvencprevcont = $row["NRVENCPREVCONT"];
            $nrapuracao     = $row["NRAPURACAO"];
            
            $nrorg = $this->environment->getNrOrg();
            
            $this->apuracaoFaturamentoService->executarProcedureFctCancFaturaContratoApu($nrseqcontrato, $nrvencprevcont, $nrapuracao, $nrorg);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     *Gera o Faturamento aparti da consulta do WebService do HCM
     */
     public function gerarEspelhoFaturamento(Request\Row $request, Response $response) {
        try {
            $row               = $request->getRow();
            $nrorg             = $this->environment->getNrOrg();
            $cdoperadorlogado  = $this->environment->getCdOperador();
            $nrseqcontrato     = $row["NRSEQCONTRATO"];
            $nrvencprevcont    = $row["NRVENCPREVCONT"];
            $dtinicompet       = $row["DTINICOMPET"];
            $dtfincompet       = $row["DTFINCOMPET"];
            $mescompet         = $row["MESCOMPET"];
            $dadosDasPessoas   = [];
            
            $this->apuracaoFaturamentoService->deleteTabelaTemporaria($nrvencprevcont, $nrseqcontrato, $nrorg);
            $this->apuracaoFaturamentoService->excluirTabelaRelatorioMedicaoFaturamento($nrseqcontrato, $nrvencprevcont, $nrorg);
            
            $nritemocontrato   = $this->apuracaoFaturamentoService->getItenDoContrato($nrseqcontrato, $nrvencprevcont, $nrorg);
            $pessoasDoContrato = $this->apuracaoFaturamentoService->getCPFpessoas($nrvencprevcont, $dtinicompet, $dtfincompet, $nrorg);
            foreach ($pessoasDoContrato as $pessoa) {
                $this->apuracaoFaturamentoService->gerarFaturamentoHora($nrseqcontrato, $nritemocontrato[0]["NRITEMOCONTRATO"], $pessoa["NRCPFPESSOA"], $pessoa["NRPARCNEPESSOA"], $mescompet, $nrvencprevcont, $nrorg);
            }
            
            $this->apuracaoFaturamentoService->calculaHoras($nrvencprevcont, $nrseqcontrato, $nrorg, $dtinicompet, $dtfincompet);
            
            //GERAR RELATORIO
            $urlRelatorio = $this->gerarRelatorioMedicaoFaturamento($nrseqcontrato, $nrvencprevcont, $nrorg, $request->getUserId());
            if (!empty($urlRelatorio)) {
                $response->addMessage(new Response\Message($urlRelatorio));
            }
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function gerarRelatorioMedicaoFaturamento($nrseqcontrato, $nrvencprevcont, $nrorg, $userId) {
        $urlRelatorio = null;
        
        $rowBirt = new Row();
        $responseBirt = new Response();
        
        $rowBirt->set("__report_name", "medicao_faturamento");
        $rowBirt->set("P_NRSEQCONTRATO", $nrseqcontrato);
        $rowBirt->set("P_NRVENCPREVCONT", $nrvencprevcont);
        $rowBirt->set("P_NRORG", $nrorg);
        
        $requestBirt = new Request\Row($rowBirt, "POST", "/geraBirtReport", $userId);
        $this->birtController->createRemoteReport($requestBirt, $responseBirt);
        
        if (isset($responseBirt->getMessages()[0])) {
            $urlRelatorio = $responseBirt->getMessages()[0]->getMessage();
        }
        
        return $urlRelatorio;
    }
	
    /**
     * Converta parte decimal da hora em minuto (depois da virgula)
     * @static
     *
     * @param $hora
     *
     * @return float
     */
    public static function converteDecimalHoraMinuto($hora){

        $aux = floor($hora);

        if ($aux === $hora){
            return $hora;
        } else {
            $decimal = $hora - $aux;
            $minuto = ($decimal * 60.0)/100.0;
            $aux += $minuto;

            return $aux;
        }
    }
    
    
     /**
     * Cancela a apuração do faturamento para o tipo "Apuração".
     */
    public function deleteTabelaTemporaria(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrvencprevcont = $row["NRVENCPREVCONT"];
            $nrseqcontrato  = $row["NRSEQCONTRATO"];
            $nrorg = $this->environment->getNrOrg();
            
            $this->apuracaoFaturamentoService->deleteTabelaTemporaria($nrseqcontrato, $nrvencprevcont, $nrorg);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function limparTabelasTemporariasFaturamento(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrseqcontrato  = $row["NRSEQCONTRATO"];
            $nrvencprevcont = $row["NRVENCPREVCONT"];
            $nrorg = $this->environment->getNrOrg();
            
            $this->apuracaoFaturamentoService->deleteTabelaTemporaria($nrvencprevcont, $nrseqcontrato, $nrorg);
            $this->apuracaoFaturamentoService->excluirTabelaRelatorioMedicaoFaturamento($nrseqcontrato, $nrvencprevcont, $nrorg);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /*
     *Busca por dados evento de tipo salario
     
    public function getEventoSalario(Request\Row $request, Response $response) {
        $nrorg             = $this->environment->getNrOrg();
        $eventos = "10_20";
        $nritemocontrato   = $this->apuracaoFaturamentoService->getEventos($nrorg, $eventos);
    }
   
     *Busca por dados evento de tipo Hora Extra
     
     public function getEventoHoraExtra(Request\Row $request, Response $response) {
        $nrorg             = $this->environment->getNrOrg();
        $eventos = "70_74_90_970_974_990";
        $nritemocontrato   = $this->apuracaoFaturamentoService->getEventos($nrorg, $eventos);
         
    }

     *Busca por dados evento de tipo Falta Injustificada
   
     public function getEventoFaltaiInjustificada(Request\Row $request, Response $response) {
        $nrorg             = $this->environment->getNrOrg();
        $eventos = "1040_1041";
        $nritemocontrato   = $this->apuracaoFaturamentoService->getEventos($nrorg, $eventos);
    }
   
     *Busca por dados evento de tipo Atraso
    
    public function getEventoAtrasos(Request\Row $request, Response $response) {
        $nrorg             = $this->environment->getNrOrg();
        $eventos = "1050";
        $nritemocontrato   = $this->apuracaoFaturamentoService->getEventos($nrorg, $eventos);
    }
    
    */
    
    
   /**
    * Rotina de Geração de Nota Antecipada.
    * 
    * Gera uma parcela do tipo "Periódico" e gera a nota fical corresponente a essa NF
    */
    
    public function gerarNotaAntecipada(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrvencprevcont   = $row["NRVENCPREVCONT"];
            $nrseqcontrato    = $row["NRSEQCONTRATO"];
            $nrgrupofatura    = $row["NRGRUPOFATURA"];
            $idfaturacontr    = $row["IDFATURACONTR"];
            $cdmodpgto        = $row["CDMODPGTO"];
            $nrapuracao       = $row["NRAPURACAO"];
            $qtdiasvencimento = $row["QTDIASVENCIMENTO"];
            $dtinicompet      = $row["DTINICOMPET"];
            $dtfincompet      = $row["DTFINCOMPET"];
            $dtvencimento     = $row["DTVENCIMENTO"];
            $vrvencimento     = $row["VRVENCIMENTO"];
            $dtemissao        = $row["DTEMISSAO"];
            $idaltusuario     = $row["IDALTUSUARIO"];
            $idsituvencprev   = $row["IDSITUVENCPREV"];
            $nragrupfatur     = $row["NRAGRUPFATUR"];
            $nrparcnegocio    = $row["NRPARCNEGOCIO"];
            $vrfaturado       = $row["VRFATURADO"];
            $vrfaltafaturar   = $row["VRFALTAFATURAR"];
            $vrnotafiscante   = $row["VRNOTAFISCANTE"];
            $nrorg            = $this->environment->getNrOrg();
            $operador         = $this->environment->getCdOperador();
            
            $this->apuracaoFaturamentoService->gerarNotaAntecipada($nrvencprevcont, $nrseqcontrato, $nrgrupofatura, $idfaturacontr, $cdmodpgto, $nrapuracao, $qtdiasvencimento, $dtinicompet, $dtfincompet, $dtvencimento, $vrvencimento, $dtemissao, $idaltusuario, $idsituvencprev, $nragrupfatur, $nrparcnegocio, $vrfaturado, $vrfaltafaturar, $vrnotafiscante,  $nrorg, $operador);
            
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /*----------------------------------------
     | --- Nova rotina de  Faturamento   --- |
     ----------------------------------------*/
    
    
     /**
     * A função "gerarFaturamentoApuracao" Realiza o Faturamento das parcelas marcadas.
     */
    public function gerarFaturamentoApuracao(Request\Row $request, Response $response) {
        set_time_limit(600);
        try {
            $row = $request->getRow();
            
            $nrfaturacont     = $row["P_NRFATURACONT"];
            $nrrateioitemvenc = $row["P_NRRATEIOITEMFAT"];
            $nritfaturacont   = $row["P_NRITFATURACONT"];
            
            $nrorg    = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();
            
            // -- Gerar Parcelas de Faturamento -- 
            $this->apuracaoFaturamentoService->gerarFaturamentoApuracao($nrfaturacont, $nritfaturacont, $nrrateioitemvenc, $nrorg, $operador);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     
    /**
     * Cancela a apuração do faturamento.
     */
    public function cancelarFaturamentoApuracao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrseqcontrato    = $row["P_NRSEQCONTRATO"];
            $dataini          = $row["P_DATAINI"];
            $datafin          = $row["P_DATAFIN"];
            $nrfaturacont     = $row["P_NRFATURACONT"];
            $nrrateioitemvenc = $row["P_NRRATEIOITEMFAT"];
            $nritfaturacont   = $row["P_NRITFATURACONT"];
            
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $this->apuracaoFaturamentoService->cancelarFaturamentoApuracao($nrseqcontrato, $dataini, $datafin, $nrfaturacont, $nrrateioitemvenc, $nrorg, $cdoperador, $nritfaturacont);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    
    /**
     * A função "anularApuracao" Realiza a anuladção do Faturamento das parcelas.
     */
    public function anularApuracao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $dsJustificativa  = $row["DSJUSTIFICATIVA"];
            $nrfaturacont     = $row["NRFATURACONT"];
            $nrSeqContrato    = $row["NRSEQCONTRATO"];
            
            $nrorg    = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();

            $this->apuracaoFaturamentoService->anularApuracao($dsJustificativa, $nrfaturacont, $nrSeqContrato, $nrorg, $operador);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     /**
     * A função "FatuManual" Realiza o Faturamento MANUAL das parcelas.
     */
    public function FatuManual(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrfaturacont     = $row["NRFATURACONT"];
            $nrSeqContrato    = $row["NRSEQCONTRATO"];
            $nrParcFatura     = $row["NRPARCFATURA"];
            $vrVencimento     = $row["VRVENCIMENTO"];
            $justificativa    = $row["DSJUSTIFICATIVA"];       
            $nrLanctoNF       = $row["NRLANCTONF"];
            $cdFilial         = $row["CDFILIAL"];
            $cdEmpresa        = $row["CDEMPRESA"];
            $vrNotaFisc       = $row["VRNOTAFISC"];
            $nrorg            = $this->environment->getNrOrg();
            $operador         = $this->environment->getCdOperador();

            $this->apuracaoFaturamentoService->FatuManual($nrfaturacont, $nrSeqContrato, $nrParcFatura, $vrVencimento, $justificativa, $nrLanctoNF, $cdFilial, $cdEmpresa, $vrNotaFisc, $nrorg, $operador);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    /**
     * A função "validaQtItens" valida a quantidade de itens da parcela.
     */
    public function validaQtItens(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrfaturacont   = $row["NRFATURACONT"];
            $nrSeqContrato  = $row["NRSEQCONTRATO"];
            $nrorg          = $this->environment->getNrOrg();

            $valida = $this->apuracaoFaturamentoService->validaQtItens($nrfaturacont, $nrSeqContrato, $nrorg);
            
            $response->addDataSet(new DataSet("VALIDA", array("QUANTIDADE" => $valida)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     /**
     * A função "desmembrarParcela" Realiza o desmembramento dos itens das parcelas.
     */
    public function desmembrarParcela(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrfaturacont          = $row["NRFATURACONT"];
            $nrSeqContrato         = $row["NRSEQCONTRATO"];
            $itDesmembramento      = $row["ITDESMEMBRAMENTO"];
            $idFormaDesmembramento = $row["IDFORMADESMEMBAMENTO"];
            $justificativa         = $row["DSJUSTIFICATIVA"];       
            $nrorg                 = $this->environment->getNrOrg();
            $operador              = $this->environment->getCdOperador();

            $this->apuracaoFaturamentoService->desmembrarParcela($nrfaturacont, $nrSeqContrato, $itDesmembramento, $idFormaDesmembramento, $justificativa, $nrorg, $operador);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function atualizarValorParcela(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrvencprevcont = $row["NRVENCPREVCONT"];
            $nrSeqContrato  = $row["NRSEQCONTRATO"];
            $nrorg          = $this->environment->getNrOrg();
            $operador       = $this->environment->getCdOperador();

            $this->apuracaoFaturamentoService->atualizarValorParcela($nrvencprevcont, $nrSeqContrato, $nrorg, $operador);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Manda validar e agrupar as parcelas para serem faturadas juntos
    **/
    public function agruparParcelasParaFaturamento(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            $nrfaturaconts = $row["NRFATURACONT"];
            $idAcao        = $row["IDACAO"];
            $nrorg    = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();
            $this->apuracaoFaturamentoService->agruparParcelasParaFaturamento($nrfaturaconts, $idAcao, $nrorg, $operador);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Manda desagrupar as parcelas ou contratos para serem faturadas
    **/
    public function desagruparParcelasFaturamento(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            $nrfaturacont   = $row["NRFATURACONT"];
            $idAcao         = $row["IDACAO"] ;
            $nrseqcontratos = $row["NRSEQCONTRATO"] ;
            $nrorg    = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();
                
            $this->apuracaoFaturamentoService->desagruparParcelasFaturamento($nrfaturacont, $nrseqcontratos, $idAcao, $nrorg, $operador);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * desagrupa um contrato de todas as parcelas ainda pendentes no faturamento
    **/
    public function retirarContratoAgrupamentoFat(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            $nragrupamento  = $row["NRAGRUPAMENTO"];
            $nrseqcontratos = $row["NRSEQCONTRATO"] ;
            $nrorg    = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();
            $this->apuracaoFaturamentoService->retirarContratoAgrupamentoFat($nragrupamento, $nrseqcontratos, $nrorg, $operador);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /*
    * Mostra o relatorio de espelho do faturamento antigo
    **/
    public function mostrarRelatorioEspelhoFaturamento(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            $nrvencprevcont = $row["NRVENCPREVCONT"];
            $nrseqcontrato  = $row["NRSEQCONTRATO"];
            $nrorg          = $this->environment->getNrOrg();

            if(!$this->apuracaoFaturamentoService->getPossuiRelatorioEspelho($nrseqcontrato, $nrvencprevcont, $nrorg))
                throw new \Exception("Operação bloqueada!<br>Apuração não efetuada. Realize-a e tente novamente.");
                
            $urlRelatorio = $this->gerarRelatorioMedicaoFaturamento($nrseqcontrato, $nrvencprevcont, $nrorg, $request->getUserId());
            if (!empty($urlRelatorio)) 
                $response->addMessage(new Response\Message($urlRelatorio));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    
    public function validarAgrupamentoFat(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $nrparcfatura  = $row["NRPARCFATURA"];
            $nrcontratofct = $row["NRCONTRATOFCT"];
            $nrfaturacont  = $row["NRFATURACONT"];
            $dtinicio      = $row["DTINICIO"];
            $dtfim         = $row["DTFIM"];

            $nrorg = $this->environment->getNrOrg();
            
            $dados = $this->apuracaoFaturamentoService->validarAgrupamentoFat($nrseqcontrato, $nrparcfatura, $nrcontratofct, $nrfaturacont, $dtinicio, $dtfim, $nrorg);
            
            $response->addDataSet(new DataSet("dados", $dados));
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     public function validarAgrupamentoFatMarcados(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            
            $vencimentosMarcados =  $row["VENCIMENTOS"];
            
            $nrorg = $this->environment->getNrOrg();
            
            $dados = $this->apuracaoFaturamentoService->validarAgrupamentoFatMarcados($vencimentosMarcados, $nrorg);
            
            $response->addDataSet(new DataSet("retorno", array("dados" => $dados)));
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Valida se existem contratos para o período que não tiveram reajuste.
    **/
    public function validarReajuste(Request\Row $request, Response $response){
        try {

            $nrorg = $this->environment->getNrOrg();
            
            $dados = $this->apuracaoFaturamentoService->validarReajuste($nrorg);
            
            $response->addDataSet(new DataSet("dados", $dados));
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function alterarStatusParcelasZeradas(Request\Row $request, Response $response) {
        try {
            $row        = $request->getRow();
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $nrfaturacont = $row["NRFATURACONT"];
            $situacao = $row["IDSITUACAO"];
            $this->apuracaoFaturamentoService->alterarStatusParcelasZeradas($nrfaturacont, $nrorg, $cdoperador, $situacao);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Altera na tabela de nota fiscal a situação e a data de saída da nota
    **/
    public function alterarDataSaidaDaNotaFiscal(Request\Row $request, Response $response) {
        try {
            $row        = $request->getRow();
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $this->apuracaoFaturamentoService->alterarDataSaidaDaNotaFiscal($row, $nrorg, $cdoperador);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}

?>